import os
from .ignore import dir_ignore, files_ignore
import shutil
from pathlib import Path

def ready(current_dir, storage, house):
  directory = current_dir
  temp = os.path.join(storage, house, 'ready')
  created_directory = set()
  for root, dirs, files in os.walk(directory):
    dirs[:] = [d for d in dirs if d not in dir_ignore]
    for file in files:
      if file in files_ignore:
        continue

      file_path = os.path.join(root, file)
      parent = Path(directory)
      child = Path(root)
      relative = child.relative_to(parent)
      folder = os.path.join(temp, str(relative))
      if relative not in created_directory:
        os.makedirs(folder, exist_ok=True)
        created_directory.add(relative)
      shutil.copy2(file_path, folder)


import os
import hashlib
import pickle
import shutil

def if_directory_empty(directory):
  if not os.path.isdir(directory):
    print(f"Error: '{directory}' is not a valid directory.")
    return False

  with os.scandir(directory) as it:
    return next(it, None) is None

def empty_ready_folder(directory):
  shutil.rmtree(directory)
  os.makedirs(directory)

def snapshot(current_directory, directory, current_house):
  working_directory = os.path.join(directory, current_house)
  ready_directory = os.path.join(working_directory, 'ready')
  snapshot_directory = os.path.join(working_directory, 'snapshot')
  if not if_directory_empty(ready_directory):
    snapshot_hash = hashlib.sha256()
    snapshot_data = {'files': {}}

    for root, dirs, files in os.walk(ready_directory):
      for file in files:
        file_path = os.path.join(root, file)

        with open(file_path, 'rb') as f:
          content = f.read()
          snapshot_hash.update(content)
          file_path = file_path.replace(ready_directory, current_directory)
          snapshot_data['files'][file_path] = content

    hash_digest = snapshot_hash.hexdigest()
    snapshot_data['file_list'] = list(snapshot_data['files'].keys())
    current_ver = current_version(snapshot_directory)
    hash_path = os.path.join(snapshot_directory, current_ver, hash_digest)
    if not os.path.exists(hash_path):
      save_at = os.path.join(snapshot_directory, str(update_version(snapshot_directory)))
      os.makedirs(save_at)
      save_file = os.path.join(save_at, hash_digest)
      with open(save_file, 'wb') as f:
        pickle.dump(snapshot_data, f)
      print(f'Snapshot created with hash {hash_digest}')
    else:
      print(f'Snapshot already exists')
    empty_ready_folder(ready_directory)
  else:
    print("Nothing to Snapshot")

def current_version(snapshot_directory):
  version_file_path = os.path.join(snapshot_directory, 'version.txt')
  version_file = open(version_file_path, 'r')
  version = version_file.read()
  version_file.close()
  return version

def update_version(snapshot_directory):
  version = current_version(snapshot_directory)
  version = int(version) + 1
  version_file_path = os.path.join(snapshot_directory, 'version.txt')
  version_file = open(version_file_path, 'w')
  version_file.write(f'{version}')
  version_file.close()
  update_working_version(snapshot_directory)
  return version

def working_version(snapshot_directory):
  current_version_file = os.path.join(snapshot_directory, 'current_version.txt')
  version_file = open(current_version_file, 'r')
  version = version_file.read()
  version_file.close()
  return version

def update_working_version(snapshot_directory, new_version = 0):
  if new_version != 0:
    version = new_version
  else:
    version = current_version(snapshot_directory)
  current_version_file = os.path.join(snapshot_directory, 'current_version.txt')
  version_file = open(current_version_file, 'w')
  version_file.write(f'{version}')
  version_file.close()
  return version


import os
import pickle
from .ignore import dir_ignore, files_ignore
from .snapshot import update_working_version

def revert_to_snapshot(current_directory, directory, house, version):
  root_path = os.path.join(directory, house, 'snapshot')
  version_path = os.path.join(root_path, version)
  directory_list = os.listdir(current_directory)
  if os.path.exists(version_path):
    version_files = os.listdir(version_path)
    hash_location = version_files[0]
    snapshot_path = os.path.join(version_path, hash_location)
    with open(snapshot_path, 'rb') as f:
      snapshot_data = pickle.load(f)
    created_directory = set()
    for file_path, content in snapshot_data['files'].items():
      directory_name = os.path.dirname(file_path)
      if directory_name not in directory_list:
        if directory_name not in created_directory:
          os.makedirs(directory_name, exist_ok=True)
          created_directory.add(directory_name)
      created_directory.add(directory_name)

      with open(file_path, 'wb') as f:
        f.write(content)

    current_files = set()
    for root, dirs, files in os.walk(current_directory, topdown=True):
      dirs[:] = [d for d in dirs if d not in dir_ignore]
      if 'svcs' in root:
        continue
      for file in files:
        if file in files_ignore:
          continue
        current_files.add(os.path.join(root, file))

    snapshot_files = set(snapshot_data['file_list'])
    files_to_delete = current_files - snapshot_files
    removed_files = []
    if files_to_delete:
      for file_path in files_to_delete:
        os.remove(file_path)
        removed_files.append(file_path)
      print(f'Removed files: {removed_files}')
    update_working_version(root_path, version)
    print(f'Reverted to snapshot {version}')
  else:
    print('Snapshot does not exist.')


import os

def new_house(path, house_name):
  location = os.path.join(path, house_name)  # Cross-platform path
  if os.path.exists(location):
    return 'Already exists please choose other name'
  else:
    os.makedirs(location)
    generate_rooms(location)
    update_house(path, house_name)
    return f'You are at {house_name}'

def generate_rooms(path):
  snapshot = os.path.join(path, 'snapshot')  # Cross-platform path
  ready = os.path.join(path, 'ready')  # Cross-platform path
  version = os.path.join(snapshot, "version.txt")  # Cross-platform path
  current_version = os.path.join(snapshot, "current_version.txt")  # Cross-platform path
  os.makedirs(snapshot)
  os.makedirs(ready)
  with open(version, "w") as f:
    f.write("0")
  with open(current_version, "w") as f:
    f.write("0")

def current_house(directory):
  try:
    house_file = os.path.join(directory, 'house.txt')  # Cross-platform path
    house = open(house_file,'r')
    current = house.read()
    house.close()
    return current
  except:
    print("House not detected")

def update_house(path, house):
  house_file = os.path.join(path, "house.txt")  # Cross-platform path
  all_house_file = os.path.join(path, "all_house.txt")  # Cross-platform path
  with open(house_file, "w") as f:
    f.write(house)
  with open(all_house_file, "a") as f:
    f.write(f"\n{house}")

def move_house(path, house):
  house_location = os.path.join(path, house)  # Cross-platform path
  if os.path.exists(house_location):
    update_house(path, house)
    return f'You are at {house}'
  else:
    return 'House does not exists'

def all_house(path):
  all_house_file = os.path.join(path, 'all_house.txt')  # Cross-platform path
  house = open(all_house_file, 'r')
  all_houses = house.read().strip()
  house.close()
  return all_houses
